clc;
clear;


%% Simulation parameters
findfocus=0;

disp_mixfigures=0;
disp_single=1;
Fontsize=14;
Linewidth=1.5;

%% Specimen parameters
%Assumed diameter of the field of view
Dtrue=5000; %m
%Diameter of curvature of the specimen (this determines the focus distance)
Dspecimen=100000;%m
%Excentrizitt fr elliptische From
ex=1;
ey=1;

%% Fitting parameters
Msim=101; %samples in radial direction used for Zernike surface fitting
Nsim=101; %samples in azimuthal direction used for Zernike surface fitting
zernike_number=8;
%prepare wavefront error by zernike polynomials
%Auto generate the zernike numeration vectors
n_zi=zeros(1,((zernike_number)*(zernike_number+1))/2);
pos=1;
for ind=0:zernike_number-1
    n_zi(pos:pos+ind)=ind;
    pos=pos+ind+1;
end
m_zi=zeros(1,length(n_zi));
for ind=1:length(n_zi)
    m_zi(ind)=2*(ind-1)-(n_zi(ind)*(n_zi(ind)+2));
end

%% Diffraction image simulation parameters
Naperture=151;  %Number of points in the aperture plane of the sample (Referred to as Nx, Ny in the paper)
Nimage=201;     %Number of sample points in the image plane
N_airy=8; %Width of the imaging region expressed as an integer multiple of airy discs
lambda=546*10^(-6); %mm Central wavelength

%% Wavefront error
%Association list https://www.telescope-optics.net/zernike_expansion_schemes.htm
P_input_error=zeros(length(m_zi),1); %mm             
% %        P_input_error(1)=1;           %Piston          n_z=0, m_z=0
% %         P_input_error(2)=-0*10^(-5);   %Tilt            n_z=1, m_z=-1
% %         P_input_error(3)=-1*10^(-5);   %Tilt            n_z=1, m_z=1
% %         P_input_error(4)=1*10^(-6);  %Astigmatism      n_z=2, m_z=-2
% %           P_input_error(5)=-1.0*10^(-6);  %Defocus          n_z=2, m_z=0
% %          P_input_error(6)=1*10^(-6);  %Astigmatism      n_z=2, m_z=2
% %         P_input_error(7)=1*10^(-5);  %Trefoil          n_z=3, m_z=-3
% %              P_input_error(8)=0.3*10^(-6);  %Coma             n_z=3, m_z=-1
% %           P_input_error(9)=-1*10^(-6);  %Coma             n_z=3, m_z=1
           P_input_error(10)=-2*10^(-6); %Trefoil          n_z=3, m_z=3
% %          P_input_error(13)=1*10^(-6); %Spherical        n_z=4, m_z=0
%% Intensity profile perpendicular to the slit

I_weight=ones(1,Naperture);

I_weight_matrix=ones(Naperture,Naperture); %This is referred to as A(x,y) in the paper

%% Remove comment for gaussian profile

%% 2D profile
% x_gaus=linspace(-1,1,Naperture);
% y_gaus=linspace(-1,1,Naperture);
% mu_gaus=0;
% sig_gaus=0.5;
% for indx=1:Naperture
%     for indy=1:Naperture
%         I_weight_matrix(indx,indy)=exp(-((sqrt((x_gaus(indx)^2+(y_gaus(indy))^2)-mu_gaus).^2)/(2*sig_gaus.^2)));
%     end
% end

%% 1D profile
% x_gaus=linspace(-1,1,Naperture);
% mu_gaus=0;
% sig_gaus=3;
% I_weight=exp(-(((x_gaus-mu_gaus).^2)/(2*sig_gaus.^2)));







%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1 simulation of the ideal topography %%

%The first step is to create an arbitrary test surface which is
%parameterized as a zernike polynomial function.

Phi_sim=linspace(0,2*pi,Nsim);
radial_array=[0,1];

if(max(max(radial_array))==1)
    r_tempsim=linspace(radial_array(1),radial_array(end),Msim)'*(Dtrue/2)/(Dspecimen/2); %Durch die Skalierung Dtrue/Dspecimen wird der richtige radiale Ausschnitt des Einheitskreises verwendet, r_tempsim ist Einheitenlos und als Bruchteil von r=1 definiert.
end

if(min(min(radial_array))==-1)
    r_tempsim=linspace(radial_array(1),radial_array(end),Msim)'*(Dtrue/2)/(Dspecimen/2);
end

%On the unit circle each radial position is associated to an x and y
px_sim=r_tempsim*cos(Phi_sim);
py_sim=r_tempsim*sin(Phi_sim);


%Test object geometry
%Sphere/Ellipsoid
r_model=1;
pz_sim=abs(sqrt(r_model-((px_sim/ex).^2+(py_sim/ey).^2)));
pz_sim=pz_sim-max(max(pz_sim));

%Test the simulated surface form by  sphere fit
xstart=1;
ystart=1;
xend=Msim;
yend=Nsim;
step=1;

%Sphere fit
X=px_sim*(Dspecimen/2);
Y=py_sim*(Dspecimen/2);
Z=pz_sim*(Dspecimen/2);
tx=X(xstart:step:xend,ystart:step:yend);
ty=Y(xstart:step:xend,ystart:step:yend);
tz=Z(xstart:step:xend,ystart:step:yend);
xline=tx(:);
yline=ty(:);
zline=tz(:);
data_array=[xline,yline,zline];
[centersim,radiussim]=sphereFit(data_array); %nm
disp(['Radius custom sphere:',num2str(radiussim*10^(-3)),' mm'])

f_lens=radiussim*10^(-3);

figurecount=1;
figure(figurecount),clf
figurecount=figurecount+1;
mesh(tx,ty,tz*10^3)
set(gca,'FontSize',Fontsize-1)
xlabel('x/mm','interpreter','none','FontSize',Fontsize)
ylabel('y/mm','interpreter','none','FontSize',Fontsize)
zlabel('Wellenfrontprofil/m','interpreter','none','FontSize',Fontsize)
title('Profil der idealen Kugellinse als Wellenfront')

%%

y_meas_input=pz_sim(:)';
X_zernike_input=zeros(length(y_meas_input),length(m_zi));

Z_nmi=zeros(Msim,Nsim,length(m_zi));
Id_i=eye(length(m_zi),length(m_zi));
Id_i(1,1)=0;
lambda_i=0;

if(max(max(radial_array))==1)
    r_zi=linspace(radial_array(1),radial_array(end),Msim);
end

if(min(min(radial_array))==-1)
    r_zi=linspace(radial_array(1),radial_array(end),Msim);
end
phi_zi=Phi_sim;

%Calculate the value of the zernike polynom Z_nm on the raster (r_z,phi_z)
for j=1:length(m_zi)
    Z_nmi(:,:,j)=zernike_poly_r(n_zi(j),m_zi(j),r_zi,phi_zi); %This is just determining the basic zernike polynomials at the sampling points (Sttzstellen)
end

for j=1:length(m_zi)
    temp=Z_nmi(:,:,j);
    X_zernike_input(:,j)=temp(:);
end

Popt_input=(lambda_i*Id_i+X_zernike_input'*X_zernike_input)^(-1)*X_zernike_input'*y_meas_input';

Fitted_input=X_zernike_input*Popt_input;
Fitted_input=reshape(Fitted_input,[Msim,Nsim]);

figure(figurecount),clf
figurecount=figurecount+1;
subplot(2,2,1)
surf(px_sim*(Dspecimen/2)*10^(-3),py_sim*(Dspecimen/2)*10^(-3),pz_sim*(Dspecimen/2)*10^(-3),'edgecolor','none') %Hier wird mit Dspecimen skaliert, da px_sim,py_sim vom normierten r_tempsim abgeleitet sind
title('Ideal Simulated Input Surface')
xlabel('mm')
ylabel('mm')
zlabel('mm')

subplot(2,2,2)
surf(px_sim*(Dspecimen/2)*10^(-3),py_sim*(Dspecimen/2)*10^(-3),Fitted_input*(Dspecimen/2)*10^(-3),'edgecolor','none')
title('Ideal Fitted Input (employed as wavefront input to represent the sample)')
xlabel('mm')
ylabel('mm')
zlabel('mm')

subplot(2,2,3)
surf(px_sim*(Dspecimen/2)*10^(-3),py_sim*(Dspecimen/2)*10^(-3),(pz_sim-Fitted_input)*(Dspecimen/2)*10^(-3),'edgecolor','none')
title('Ideal Simulated Input Surface - simulated fitted input surface')
xlabel('mm')
ylabel('mm')
zlabel('mm')
    

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%The optic considered is a lens with rectangular aperture D and focus distance
%f, whith the object at infinity and the image at distance f
D_lens=Dtrue/1000; %mm


%% Step 2 Huygens approach for diffraction image

%Polar coordinate description of the aperture plane
rho_aperture=linspace(radial_array(1),radial_array(end),Naperture)*(Dtrue/2)/(Dspecimen/2); %Der Radius fr die Wellenfrontsimulation der Linse wird ebenfalls auf den Einheitskreisausschnitt normiert
theta_aperture=linspace(0,2*pi,Naperture);
%Cartesian coordinate description of the aperture plane
x_sim=rho_aperture'*cos(theta_aperture);
y_sim=rho_aperture'*sin(theta_aperture);

%airy_diameter_object_rayleigh=0.61*lambda/0.5;




%% Wavefront error described in the aperture plane

if(max(max(radial_array))==1)
    r_zi_aperture=linspace(radial_array(1),radial_array(end),Naperture);
end

if(min(min(radial_array))==-1)
    r_zi_aperture=linspace(radial_array(1),radial_array(end),Naperture);
end

Z_nmi_Fraunhofer=zeros(Naperture,Naperture,length(m_zi));
for j=1:length(m_zi)
    %Calculate the value of the zernike polynom Z_nm on the raster (rho_aperture,theta_aperture)
    %This polar coordinate based description is employed in the
    %Fraunhofer simulation of the PSF
    Z_nmi_Fraunhofer(:,:,j)=zernike_poly_r(n_zi(j),m_zi(j),r_zi_aperture,theta_aperture); %This is just determining the basic zernike polynomials at the sampling points (Sttzstellen)
    
end
X_zernike_input_Fraunhofer=zeros(Naperture*Naperture,length(m_zi));
for j=1:length(m_zi)
    temp=Z_nmi_Fraunhofer(:,:,j);
    X_zernike_input_Fraunhofer(:,j)=temp(:);
end

%%Extra symmetric interval calculation for zernike terms in a cut at
r_zi_aperture_cut=linspace(-1,1,Naperture);
theta_aperture_cut=0;

Z_nmi_Fraunhofer_cut=zeros(Naperture,1,length(m_zi));
for j=1:length(m_zi)
    %Calculate the value of the zernike polynom Z_nm on the raster (rho_aperture,theta_aperture)
    %This polar coordinate based description is employed in the
    %Fraunhofer simulation of the PSF
    Z_nmi_Fraunhofer_cut(:,:,j)=zernike_poly_r(n_zi(j),m_zi(j),r_zi_aperture_cut,theta_aperture_cut); %This is just determining the basic zernike polynomials at the sampling points (Sttzstellen)
    
end
X_zernike_input_Fraunhofer_cut=zeros(Naperture,length(m_zi));
for j=1:length(m_zi)
    temp=Z_nmi_Fraunhofer_cut(:,:,j);
    X_zernike_input_Fraunhofer_cut(:,j)=temp(:);
end

%%

%% Wavefront error described by zernike polynomials

P_input=P_input_error+Popt_input;
        %Diese Wellenfront enthlt inherent die Beschrnkung auf die
        %Kreisfrmige Apertur und beeinflusst die Ergebnisse der Fraunhofer
        %Beugungssimulation
        Wavefront_input_Fraunhofer=X_zernike_input_Fraunhofer*P_input;
        Wavefront_input_Fraunhofer=reshape(Wavefront_input_Fraunhofer,[Naperture,Naperture]);
        
        
        Wavefront_input_Fraunhofer_cut=X_zernike_input_Fraunhofer_cut*P_input;
        Wavefront_input_Fraunhofer_error_cut=X_zernike_input_Fraunhofer_cut*P_input_error;
        Wavefront_input_Fraunhofer_ideal_cut=X_zernike_input_Fraunhofer_cut*Popt_input;
            
        %Test the simulated surface form by  sphere fit
        xstart=1;
        ystart=1;
        xend=Naperture;
        yend=Naperture;
        step=10;
        
        %Sphere fit
        X=x_sim*(Dspecimen/2000); %mm
        Y=y_sim*(Dspecimen/2000); %mm
        Z=Wavefront_input_Fraunhofer*(Dspecimen/2000); %mm
        tx=X(xstart:step:xend,ystart:step:yend);
        ty=Y(xstart:step:xend,ystart:step:yend);
        tz=Z(xstart:step:xend,ystart:step:yend);
        xline=tx(:);
        yline=ty(:);
        zline=tz(:);
        data_array=[xline,yline,zline];
        [centersim,radiussim]=sphereFit(data_array); %mm
        disp(['Radius custom sphere with wavefront errors:',num2str(radiussim),' mm'])
        

%Durchmesser und Fokuslnge legen die Blendenzahl des Prflings und damit
%seine Beugungsbegrenzte MTF fest.

NA_object=sin(atan(D_lens*0.5/f_lens));
F_nr_object=f_lens/D_lens; %for object at infinity k=f/D
disp(['Wavelength ',num2str(lambda*10^3),' m'])
disp(['F-number ',num2str(F_nr_object)])

airy_diameter_object_rayleigh=0.61*lambda/NA_object;
airy_diameter_object_abbe=lambda/(2*NA_object);
disp(['Numerical aperure object: ',num2str(NA_object),', Airy Disc object Rayleigh: ',num2str(airy_diameter_object_rayleigh*10^3),' m', ', Airy Disc object Abbe: ',num2str(airy_diameter_object_abbe*10^3),' m'])

Xi_cutoff_lens=1/(lambda*F_nr_object); %lp/mm
disp(['Theoretical cutoff frequency object: ', num2str(Xi_cutoff_lens),' lp/mm'])

%Polar coordinate description of the image plane
w_image=linspace(-N_airy*airy_diameter_object_rayleigh,N_airy*airy_diameter_object_rayleigh,Nimage);%radial distance in mm (in the paper the coordinates in the image plane are px, py)
psi_image=linspace(0,2*pi,Nimage);


%% Cartesian description of the input wavefront %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Cartesian coordinate description of the aperture plane (in the paper the coordinates of the aperture plane are x,y)
x_aperture_scaled=linspace(-1,1,Naperture)*D_lens/2; %Der Radius fr die Wellenfrontsimulation der Linse wird ebenfalls auf den Einheitskreisausschnitt normiert
y_aperture_scaled=linspace(-1,1,Naperture)*D_lens/2;

x_aperture=linspace(-1,1,Naperture); %Der Radius fr die Wellenfrontsimulation der Linse wird ebenfalls auf den Einheitskreisausschnitt normiert
y_aperture=linspace(-1,1,Naperture);


%% Wavefront error described in the aperture plane

Z_nmi_Fraunhofer_cart=zeros(Naperture,Naperture,length(m_zi));

for j=1:length(m_zi)
Z_nmi_Fraunhofer_cart(:,:,j)=zernike_poly_xy_cartesian(n_zi(j),m_zi(j),x_aperture,y_aperture);
end

X_zernike_input_Fraunhofer_cart=zeros(Naperture*Naperture,length(m_zi));
for j=1:length(m_zi)
    temp=Z_nmi_Fraunhofer_cart(:,:,j);
    X_zernike_input_Fraunhofer_cart(:,j)=temp(:);
end

Wavefront_input_Fraunhofer_cart=X_zernike_input_Fraunhofer_cart*P_input;
Wavefront_input_Fraunhofer_cart=reshape(Wavefront_input_Fraunhofer_cart,[Naperture,Naperture]);

r_zi_aperture_cart_matrix=zeros(Naperture,Naperture);
aperture_mask=zeros(Naperture,Naperture);
for indx=1:length(x_aperture)
    for indy=1:length(y_aperture)
        r_zi_aperture_cart_matrix(indx,indy)=sqrt(x_aperture(indx).^2+y_aperture(indy).^2);
    end
end

zerovallist=find(abs(r_zi_aperture_cart_matrix)>1);
tempwave=Wavefront_input_Fraunhofer_cart(:);
tempwave(zerovallist)=0;
Wavefront_input_Fraunhofer_cart=reshape(tempwave,[Naperture,Naperture]);
Wavefront_Offset=min(min(Wavefront_input_Fraunhofer_cart));
Wavefront_input_Fraunhofer_cart=Wavefront_input_Fraunhofer_cart-Wavefront_Offset;
tempwave=Wavefront_input_Fraunhofer_cart(:);
tempwave(zerovallist)=0;
Wavefront_input_Fraunhofer_cart=-1*reshape(tempwave,[Naperture,Naperture]);

Wavefront_input_Fraunhofer_cart_error=X_zernike_input_Fraunhofer_cart*P_input_error;
Wavefront_input_Fraunhofer_cart_error=reshape(Wavefront_input_Fraunhofer_cart_error,[Naperture,Naperture]);

Wavefront_input_Fraunhofer_cart_error_cut=Wavefront_input_Fraunhofer_cart_error(:,round(Naperture/2));


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Determine frequency axis
deltax=mean(diff(w_image)); %mm
deltaf=1/(Nimage*deltax); %1/mm
f_axis=linspace(0,1,Nimage)*deltaf*(Nimage); %lp/mm

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Determine Diffraction pattern based on huygens principle

rho_aperture_sym=linspace(-D_lens/2,D_lens/2,Naperture);


for search_ind=1:1

    
%     Distance=f_lens;
    
if(search_ind==1)
    Distance=linspace(f_lens-1,f_lens+1,20);
else
    Distance=linspace(Distance(foc_ind-1),Distance(foc_ind+1),20);
end

%Propagation vector
r_image=zeros(Naperture,Nimage,length(Distance));
for ind_ap=1:Naperture
    for ind_im=1:Nimage
        for ind_dis=1:length(Distance)
            r_image(ind_ap,ind_im,ind_dis)=sqrt((-rho_aperture_sym(ind_ap)+w_image(ind_im)).^2+Distance(ind_dis).^2);
        end
    end
end

Wavefront_input_Fraunhofer_image=zeros(Naperture,Nimage,length(Distance));
Wavefront_input_Fraunhofer_image_ideal=zeros(Naperture,Nimage,length(Distance));


for ind_ap=1:Naperture
    for ind_im=1:Nimage
        for ind_dis=1:length(Distance)
            Wavefront_input_Fraunhofer_image(ind_ap,ind_im,ind_dis)=Wavefront_input_Fraunhofer_cut(ind_ap)*(Dspecimen/2000);
            Wavefront_input_Fraunhofer_image_ideal(ind_ap,ind_im,ind_dis)=Wavefront_input_Fraunhofer_ideal_cut(ind_ap)*(Dspecimen/2000);
        end
    end
end

E_image=exp(-i*(2*pi/lambda)*(r_image+Wavefront_input_Fraunhofer_image))./r_image;

%Apply weighting function for intensity
E_image=I_weight'.*E_image;

E_image_coherent=squeeze(sum(E_image,1));
I_image_coherent=E_image_coherent.*conj(E_image_coherent);

%% For ideal image
E_image_ideal=exp(-i*(2*pi/lambda)*(r_image+Wavefront_input_Fraunhofer_image_ideal))./r_image;

%Apply weighting function for intensity
%E_image_ideal=I_weight'.*E_image_ideal;

E_image_coherent_ideal=squeeze(sum(E_image_ideal,1));
I_image_coherent_ideal=E_image_coherent_ideal.*conj(E_image_coherent_ideal);

%% Calculate MTF for all positions
MTF_all=abs(fft(I_image_coherent));
MTF_all=MTF_all./MTF_all(1,:);

f_ind=min(find(f_axis>30));
lpeval=f_axis(f_ind);

%% Determine focus position by MTF
if(findfocus==1)
    [foc_val,foc_ind]=max(MTF_all(f_ind,:));
else
    foc_ind=round(length(Distance)/2);
end

%% Extract the LSF at the focus position
%[foc_val,foc_ind]=max(max(I_image_coherent));
z_focus=Distance(foc_ind);

if(length(Distance)==1)
    LSF_foc=I_image_coherent';
else
    LSF_foc=I_image_coherent(:,foc_ind);
end

%Calculate the MTF at focus point
MTF_foc=abs(fft(LSF_foc));
MTF_foc=MTF_foc/MTF_foc(1);


end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Beugungsbegrenzte MTF
nu=linspace(0,2,100);
nu_scaled=linspace(0,Xi_cutoff_lens,100);
MTF_bb=(1/pi)*(2*acos(nu/2)-nu.*sqrt(1-(nu/2).^2));

%Fraunhofer solution rectangular aperture (Introduction to Fourier Optics Goodman 4.4.1 p.75 ff)
I_xy_FH=((sin(2*pi*(D_lens/2)*(w_image+1*10^(-9))/(lambda*f_lens))./(2*pi*(D_lens/2)*(w_image+1*10^(-9))/(lambda*f_lens))).^2)'*((sin(2*pi*(D_lens/2)*(w_image+1*10^(-9))/(lambda*f_lens))./(2*pi*(D_lens/2)*(w_image+1*10^(-9))/(lambda*f_lens))).^2);

I_xy_FH_1D=((sin(2*pi*(D_lens/2)*(w_image+1*10^(-9))/(lambda*f_lens))./(2*pi*(D_lens/2)*(w_image+1*10^(-9))/(lambda*f_lens))).^2);

MTF_FH_1D=abs(fft(I_xy_FH_1D));
MTF_FH_1D=MTF_FH_1D/MTF_FH_1D(1);


%% FFT Simulation

% Aperture input
%The inner aperture has dimensions of D_lens x D_lens, while the whole
%aperture has the dimensions of (Mscale+1)*D_lens x (Mscale+1)*D_lens (Use a rectangular aperture for a cartesian simulation)
Mscale=50;
ap_input=zeros(Naperture*(Mscale+1),Naperture*(Mscale+1));
ap_input(Naperture*round(Mscale/2)+1:Naperture*(round(Mscale/2)+1),Naperture*round(Mscale/2)+1:Naperture*(round(Mscale/2)+1))=I_weight_matrix.*exp(-i*(2*pi/(lambda))*(-1)*Wavefront_input_Fraunhofer_cart_error*(Dspecimen/2000));

dx_input=D_lens/Naperture; %mm
dx_output=1/((Mscale+1)*Naperture*dx_input); % lp/mm

x_axis_fft=-((Mscale+1)/2)*D_lens+dx_input/2:dx_input:((Mscale+1)/2)*D_lens-dx_input/2;
y_axis_fft=-((Mscale+1)/2)*D_lens+dx_input/2:dx_input:((Mscale+1)/2)*D_lens-dx_input/2;

% Determine the correct scale of the axis in the image plane
pixX=(Mscale+1)*Naperture;
dx2=abs(f_lens*lambda/(pixX*dx_input));
w_image_fft=dx2*(x_axis_fft/dx_input);

%Calculate the FFT

image_output=abs(fftshift(fft2(ap_input))).^2;
image_output=image_output/max(max(image_output));

LSF_FFT=sum(image_output,2);
LSF_FFT=LSF_FFT/max(LSF_FFT);

%Interpolate the LSF_FFT to the same axis as the diffraction models (w_image)
LSF_FFT_interp = interp1(w_image_fft,LSF_FFT,w_image);

MTF_FFT_interp=abs(fft(LSF_FFT_interp));
MTF_FFT_interp=MTF_FFT_interp/MTF_FFT_interp(1);

figure(figurecount),clf
figurecount=figurecount+1;
imagesc(x_axis_fft,y_axis_fft,abs(ap_input))
axis equal

figure(figurecount),clf
figurecount=figurecount+1;
imagesc(image_output)
axis equal

figure(figurecount),clf
figurecount=figurecount+1;
plot(w_image_fft,LSF_FFT)
hold on
plot(w_image,LSF_FFT_interp,'--')


%% Fraunhofer Model
% % % %Try recieving the PSF from Fraunhover diffraction Born p.395 and Proc. of
% % % %SPIE Vol. 11062 1106214-4 eq. (5)
s_born=f_lens; %Distance to the screen in mm (Pure scaling factor for screen size)
U_p_born_cartesian=zeros(Nimage,Nimage);
U_p_born_cartesian1D=zeros(Nimage,1);

[Wmat1,Wmat2] = meshgrid(w_image,w_image);

[Amat1,Amat2] = meshgrid(x_aperture_scaled,y_aperture_scaled);


for kind=1:Nimage
    for lind=1:Nimage
    %Das Fraunhofer Beugungsintegral/Summe fr die kreisfrmige/rechteckige Apertur stellt quasi die Summe ber
    %alle Elementarwellenfelder aus der Aperturebene in einem Punkt
    %in der Bildebene dar, unter
    %der Annahme, dass eine ideale Ebene Welle auf die Apertur
    %auftritt. Wellenfrontfehler in der Aperturebene werden durch einen
    %separaten term beschrieben, wie in SPIE Vol. 11062 1106214-4 eq.
    %(5) beschrieben.

    %Wmat1, Wmat2 are rectangles on the screen in the image plane
    %Amat1, Amat2 are rectangles in the aperture plane
    U_p_born_cartesian(kind,lind)=sum(sum(I_weight_matrix.*exp(-i*(2*pi/lambda)*((1/s_born)*(Wmat1(kind,lind)*Amat1+Wmat2(kind,lind)*Amat2)-Wavefront_input_Fraunhofer_cart_error*(Dspecimen/2000)))));
    
    end
    
    U_p_born_cartesian1D(kind)=sum(exp(-i*(2*pi/lambda)*((1/s_born)*(w_image(kind)*rho_aperture_sym)-Wavefront_input_Fraunhofer_error_cut'*(Dspecimen/2000))));
end

I_born1D=U_p_born_cartesian1D.*conj(U_p_born_cartesian1D);

MTF_born1D=abs(fft(I_born1D));
MTF_born1D=MTF_born1D/max(MTF_born1D);
%%

I_born=U_p_born_cartesian.*conj(U_p_born_cartesian);

MTF_born=abs(fft(I_born));
MTF_born=MTF_born/max(MTF_born);
%%

line=zeros(Nimage,Nimage);
line(:,round(Nimage/2))=1;

LSF_born=sum(I_born,2);
LSF_born=LSF_born/max(LSF_born);

LSF_born2=conv2(abs(I_born)',line,'same');
LSF_born2=LSF_born2(round(Nimage/2),:);
LSF_born2=LSF_born2./max(LSF_born2);

PSF_born_cut=I_born(:,round(Nimage/2));
PSF_born_cut=PSF_born_cut./max(PSF_born_cut);

MTF_LSF_born=abs(fft(LSF_born));
MTF_LSF_born=MTF_LSF_born/max(MTF_LSF_born);

MTF_LSF_born2=abs(fft(LSF_born2));
MTF_LSF_born2=MTF_LSF_born2/max(MTF_LSF_born2);

MTF_PSF_born_cut=abs(fft(PSF_born_cut));
MTF_PSF_born_cut=MTF_PSF_born_cut/max(MTF_PSF_born_cut);

%%


%% Figures

if (disp_mixfigures==1)

figure(figurecount),clf
figurecount=figurecount+1;
surf(w_image,w_image,I_born./max(max(I_born)),'edgecolor','none')
xlabel('y/mm)')
ylabel('x/mm)')
zlabel('PSF/1')
title(['Fraunhofer PSF at focus z=',num2str(f_lens),' mm'])

figure(figurecount),clf
figurecount=figurecount+1;
plot(w_image,(LSF_foc./max(LSF_foc)),'-x')
hold on
plot(w_image,(I_born1D./max(I_born1D)),'-o')
plot(w_image,I_xy_FH(:,round(Nimage/2)),'-d')
plot(w_image,PSF_born_cut,'*')
plot(w_image,LSF_born,'-')
plot(w_image,LSF_born2,'--')
xlabel('x/mm')
ylabel('PSF/1')
title(['PSF at focus point z=',num2str(f_lens),' mm'])
legend({'1D Huygens','1D Fraunhofer','Analytical','BC','LSF Fraunhofer','LSF2'})

figure(figurecount),clf
figurecount=figurecount+1;
plot(f_axis,MTF_foc,'x')
hold on
plot(f_axis,MTF_born1D,'o')
plot(f_axis,MTF_FH_1D,'d')
plot(f_axis,MTF_PSF_born_cut,'*')
plot(f_axis,MTF_LSF_born,'-')
plot(f_axis,MTF_LSF_born2,'--')
xlim([0,1.5*Xi_cutoff_lens])
xlabel('f/(lp/mm)')
ylabel('MTF/1')
title(['MTF at focus point z=',num2str(f_lens),' mm'])
legend('1D Huygens','1D Fraunhofer','Analytical','BC','LSF Fraunhofer','LSF2')

figure(figurecount),clf
figurecount=figurecount+1;
plot(f_axis,MTF_foc-MTF_FH_1D','x')
hold on
plot(f_axis,MTF_born1D-MTF_FH_1D','o')
plot(f_axis,MTF_FH_1D-MTF_FH_1D,'d')
plot(f_axis,MTF_PSF_born_cut-MTF_FH_1D','*')
plot(f_axis,MTF_LSF_born-MTF_FH_1D','-')
plot(f_axis,MTF_LSF_born2-MTF_FH_1D,'--')
xlim([0,1.5*Xi_cutoff_lens])
xlabel('f/(lp/mm)')
ylabel('MTF/1')
title(['MTF difference to Ana at focus point z=',num2str(f_lens),' mm'])
legend('1D Huygens','1D Fraunhofer','Analytical','BC','LSF Fraunhofer','LSF2')

figure(figurecount),clf
figurecount=figurecount+1;
plot(x_aperture_scaled,Wavefront_input_Fraunhofer_error_cut*(Dspecimen/2000)*1000)
hold on
plot(x_aperture_scaled,Wavefront_input_Fraunhofer_cart_error_cut*(Dspecimen/2000)*1000,'x')
title('wavefront error in x')
xlabel('x/mm')
ylabel('Wavefront error/m')


figure(figurecount),clf
figurecount=figurecount+1;
surf(x_aperture_scaled,y_aperture_scaled,Wavefront_input_Fraunhofer_cart_error*(Dspecimen/2000)*1000,'edgecolor','none')
title('wavefront error')
xlabel('y/mm')
ylabel('x/mm')
zlabel('Wavefront error/m')

end



if (disp_single==1)
    N_airy_disp=round(2*N_airy/3);
    
    %% Analytical
    figure(figurecount),clf
    figurecount=figurecount+1;
    surf(w_image,w_image,I_xy_FH./max(max(I_xy_FH)),'edgecolor','none')
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    ylim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('y/mm)','FontSize',Fontsize)
    ylabel('x/mm)','FontSize',Fontsize)
    zlabel('PSF/1','FontSize',Fontsize)
    title(['Analytical Fraunhofer PSF at focus z=',num2str(f_lens),' mm'])
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(w_image,I_xy_FH(:,round(Nimage/2)),'-d','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('PSF/1','FontSize',Fontsize)
    title(['PSF at focus point z=',num2str(f_lens),' mm'])
    legend({'Analytical'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,MTF_FH_1D,'d','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('MTF/1','FontSize',Fontsize)
    title(['MTF at focus point z=',num2str(f_lens),' mm'])
    legend('Analytical')
    
    %% 1D Huygens
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(w_image,LSF_foc./max(LSF_foc),'x','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('PSF/1','FontSize',Fontsize)
    title(['PSF at focus point z=',num2str(f_lens),' mm'])
    legend({'1D Huygens'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,MTF_foc,'x','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('MTF/1','FontSize',Fontsize)
    title(['MTF at focus point z=',num2str(f_lens),' mm'])
    legend({'1D Huygens'})
    
    %% 1D/2D+LSF Fraunhofer
    figure(figurecount),clf
    figurecount=figurecount+1;
    surf(w_image,w_image,I_born./max(max(I_born)),'edgecolor','none')
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    ylim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('y/mm)','FontSize',Fontsize)
    ylabel('x/mm)','FontSize',Fontsize)
    zlabel('PSF/1','FontSize',Fontsize)
    title(['Fraunhofer PSF at focus z=',num2str(f_lens),' mm'])
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(w_image,(I_born1D./max(I_born1D)),'o','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('PSF/1','FontSize',Fontsize)
    title(['PSF at focus point z=',num2str(f_lens),' mm'])
    legend({'1D Fraunhofer'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,MTF_born1D,'o','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('MTF/1','FontSize',Fontsize)
    title(['MTF at focus point z=',num2str(f_lens),' mm'])
    legend({'1D Fraunhofer'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(w_image,LSF_born,'-','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('LSF/1','FontSize',Fontsize)
    title(['LSF at focus point z=',num2str(f_lens),' mm'])
    legend({'LSF Fraunhofer'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,MTF_LSF_born,'-','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('MTF/1','FontSize',Fontsize)
    title(['MTF at focus point z=',num2str(f_lens),' mm'])
    legend({'LSF Fraunhofer'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(w_image,LSF_FFT_interp,'--','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('LSF/1','FontSize',Fontsize)
    title(['LSF at focus point z=',num2str(f_lens),' mm'])
    legend({'LSF FFT'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,MTF_FFT_interp,'--','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('MTF/1','FontSize',Fontsize)
    title(['MTF at focus point z=',num2str(f_lens),' mm'])
    legend({'LSF FFT'})
    
    
    %% Mixed plots
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(w_image,I_xy_FH(:,round(Nimage/2)),'-d','Linewidth',Linewidth)
    hold on
    plot(w_image,LSF_foc./max(LSF_foc),'x','Linewidth',Linewidth)
    plot(w_image,(I_born1D./max(I_born1D)),'o','Linewidth',Linewidth)
    plot(w_image,LSF_born,'-','Linewidth',Linewidth)
    %plot((1/sqrt(2))*(x_axis_image-0.5*dx_input/f_lens),LSF_FFT,'--')
    %plot(x_axis_image-0.000283,LSF_FFT,'--')
    plot(w_image,LSF_FFT_interp,'--','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([-N_airy_disp*airy_diameter_object_rayleigh,N_airy_disp*airy_diameter_object_rayleigh])
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('PSF/1','FontSize',Fontsize)
    title(['PSF at focus point z=',num2str(f_lens),' mm'])
    legend({'Analytical','1D Huygens','1D Fraunhofer','LSF Fraunhofer','LSF FFT'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,MTF_FH_1D,'d','Linewidth',Linewidth)
    hold on
    plot(f_axis,MTF_foc,'x','Linewidth',Linewidth)
    plot(f_axis,MTF_born1D,'o','Linewidth',Linewidth)
    plot(f_axis,MTF_LSF_born,'-','Linewidth',Linewidth)
    plot(f_axis,MTF_FFT_interp,'--','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1,'Linewidth',Linewidth)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('MTF/1','FontSize',Fontsize)
    title(['MTF at focus point z=',num2str(f_lens),' mm'])
    legend({'Analytical','1D Huygens','1D Fraunhofer','LSF Fraunhofer','LSF FFT'})
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,(MTF_FH_1D-MTF_FH_1D)*100,'d','Linewidth',Linewidth)
    hold on
    plot(f_axis,(MTF_foc-MTF_FH_1D')*100,'x','Linewidth',Linewidth)
    plot(f_axis,(MTF_born1D-MTF_FH_1D')*100,'o','Linewidth',Linewidth)
    plot(f_axis,(MTF_LSF_born-MTF_FH_1D')*100,'-','Linewidth',Linewidth)
    plot(f_axis,(MTF_FFT_interp-MTF_FH_1D)*100,'--','Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('f/(lp/mm)','FontSize',Fontsize)
    ylabel('\DeltaMTF/%','FontSize',Fontsize)
    title(['MTF Difference at focus point z=',num2str(f_lens),' mm'])
    legend({'Analytical','1D Huygens','1D Fraunhofer','LSF Fraunhofer','LSF FFT'},'Location','southeast')
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(x_aperture_scaled,Wavefront_input_Fraunhofer_error_cut*(Dspecimen/2000)*10^6,'Linewidth',Linewidth)
    set(gca,'FontSize',Fontsize-1)
    title('Wavefront error in x-direction')
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('Wavefront error/nm','FontSize',Fontsize)
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    surf(x_aperture_scaled,y_aperture_scaled,Wavefront_input_Fraunhofer_cart_error*(Dspecimen/2000)*10^6,'edgecolor','none')
    set(gca,'FontSize',Fontsize-1)
    title('Wavefront error')
    xlabel('y/mm','FontSize',Fontsize)
    ylabel('x/mm','FontSize',Fontsize)
    zlabel('Wavefront error/nm','FontSize',Fontsize)
    
    
    figure(figurecount),clf
    figurecount=figurecount+1;
    imagesc(w_image,Distance,I_image_coherent')
    title('Through focus 1D Huygens')
    xlabel('x/mm','FontSize',Fontsize)
    ylabel('z/mm','FontSize',Fontsize)
    
    
    
end

%% For paper

    figure(figurecount),clf
    figurecount=figurecount+1;
    plot(f_axis,(MTF_FH_1D-MTF_FH_1D)*100,'d','Linewidth',Linewidth)
    hold on
    plot(f_axis,(MTF_foc-MTF_FH_1D')*100,'x','Linewidth',Linewidth)
    plot(f_axis,(MTF_born1D-MTF_FH_1D')*100,'o','Linewidth',Linewidth)
    plot(f_axis,(MTF_LSF_born-MTF_FH_1D')*100,'-','Linewidth',Linewidth)
    plot(f_axis,(MTF_FFT_interp-MTF_FH_1D)*100,'--','Linewidth',Linewidth)
    xlim([0,1.5*Xi_cutoff_lens])
    xlabel('$f$/(lp/mm)','interpreter','none')
    ylabel('$\Delta$MTF/$\%$','interpreter','none')
    legend({'Analytical','1D Huygens','1D Fraunhofer','LSF Fraunhofer','LSF FFT'},'Location','southeast')




